Built-in conditions.
Structs
- All
- True when every inner condition is true. An empty list is true.
- Always
- A fixed boolean answer. The default is
true. - Any
- True when any inner condition is true. An empty list is false.
- Check
- A condition created from a closure.
- Flag
- Reads a chain flag. Outside a chain, it evaluates to
false. - Not
- Inverts a condition. A missing condition evaluates to
false.
Functions
- all
- Creates a condition that requires every supplied condition.
- any
- Creates a condition that accepts any supplied condition.
- check
- Wraps a closure as a condition.
- flag
- Creates a condition that expects the flag to be set.
- flag_
clear - Creates a condition that expects the flag to be clear.
- not
- Creates an inverted condition.